<?php

namespace Ebuy\Discount\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Ebuy\Discount\Http\Requests\DiscountRequest;
use Ebuy\Discount\Interfaces\DiscountRepositoryInterface;
use Ebuy\Discount\Models\Discount;

/**
 * Resource controller class for discount.
 */
class DiscountResourceController extends BaseController
{

    /**
     * Initialize discount resource controller.
     *
     * @param type DiscountRepositoryInterface $discount
     *
     * @return null
     */
    public function __construct(DiscountRepositoryInterface $discount)
    {
        parent::__construct();
        $this->repository = $discount;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Ebuy\Discount\Repositories\Criteria\DiscountResourceCriteria::class);
    }

    /**
     * Display a list of discount.
     *
     * @return Response
     */
    public function index(DiscountRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Ebuy\Discount\Repositories\Presenter\DiscountPresenter::class)
                ->$function();
        }

        $discounts = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('discount::discount.names'))
            ->view('discount::discount.index', true)
            ->data(compact('discounts', 'view'))
            ->output();
    }

    /**
     * Display discount.
     *
     * @param Request $request
     * @param Model   $discount
     *
     * @return Response
     */
    public function show(DiscountRequest $request, Discount $discount)
    {

        if ($discount->exists) {
            $view = 'discount::discount.show';
        } else {
            $view = 'discount::discount.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('discount::discount.name'))
            ->data(compact('discount'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new discount.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(DiscountRequest $request)
    {

        $discount = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('discount::discount.name')) 
            ->view('discount::discount.create', true) 
            ->data(compact('discount'))
            ->output();
    }

    /**
     * Create new discount.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(DiscountRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $discount                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('discount::discount.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('discount/discount/' . $discount->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/discount/discount'))
                ->redirect();
        }

    }

    /**
     * Show discount for editing.
     *
     * @param Request $request
     * @param Model   $discount
     *
     * @return Response
     */
    public function edit(DiscountRequest $request, Discount $discount)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('discount::discount.name'))
            ->view('discount::discount.edit', true)
            ->data(compact('discount'))
            ->output();
    }

    /**
     * Update the discount.
     *
     * @param Request $request
     * @param Model   $discount
     *
     * @return Response
     */
    public function update(DiscountRequest $request, Discount $discount)
    {
        try {
            $attributes = $request->all();

            $discount->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('discount::discount.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('discount/discount/' . $discount->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('discount/discount/' . $discount->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the discount.
     *
     * @param Model   $discount
     *
     * @return Response
     */
    public function destroy(DiscountRequest $request, Discount $discount)
    {
        try {

            $discount->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('discount::discount.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('discount/discount/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('discount/discount/' . $discount->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple discount.
     *
     * @param Model   $discount
     *
     * @return Response
     */
    public function delete(DiscountRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('discount::discount.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('discount/discount'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/discount/discount'))
                ->redirect();
        }

    }

    /**
     * Restore deleted discounts.
     *
     * @param Model   $discount
     *
     * @return Response
     */
    public function restore(DiscountRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('discount::discount.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/discount/discount'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/discount/discount/'))
                ->redirect();
        }

    }

}
